/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Thu Sep 19 00:39:02 HKT 2002
    copyright            : (C) 2002 by Andrew D. Balsa
    email                : andrebalsa@mailingaddress.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdlib>
#include <iostream>
#include <string>

using namespace std;

/*  main.cpp
 *  Initializes and interfaces with the Go playing engine
 *  This program is based on the original wally program
 */

#include "main.h"
#include "game.h"

extern int gtp();
extern game thegame;

extern void fatal(char *);
extern void panic(char *);

extern int mymove();
extern int enemymove();

extern void initgame();
extern void score();

/***************************************************************************
 *                                                                         *
 *                              Main Function                             *
 *                                                                         *
 ***************************************************************************
 */

int main( int argc, char *argv[] )
{
  bool playgtp = false;        // assume we are playing on the console

  thegame.wallyColor = BLACK;  // Wallyplus plays Black by default

  thegame.boardSize = 9;       // Size of board is 9x9 by default

  thegame.blackHandicap = 0;   // number of handicap stones granted to Black

  thegame.komi = 0;            // komi for this game

    
  argv++;            /* skip program name */

/* Parse command line arguments */

  for( ; --argc; ++argv) {

    // GTP mode ?
    if(0==strcmp(*argv,"-gtp")) {
      playgtp = true;
      break;    // no need to parse any other arguments
    }

    // Board size ?
    if (0==strcmp(*argv, "-s")) {
      ++argv;
      if (1==sscanf(*argv, "%d", &thegame.boardSize)) {
        if(thegame.boardSize < 9 || thegame.boardSize > 19)
          fatal("board size out of range on input line");
        --argc;
      }
      else
        fatal("board size not found");
    }

    // Black handicap ?
    if (0==strcmp(*argv, "-handicap")) {
      ++argv;
      if (1==sscanf(*argv, "%d", &thegame.blackHandicap)) {
        if(thegame.blackHandicap < 2 || thegame.blackHandicap > 17)
             fatal("handicap out of range on input line");
        --argc;
      }
      else
        fatal("handicap not specified");
    }
     // Komi ?
    if (0==strcmp(*argv, "-komi")) {
      ++argv;
      if (1==sscanf(*argv, "%f", &thegame.komi)){
        --argc;
      }
      else
        fatal("komi not specified");
    }
      
    // Wally plays white ?
    else if(0==strcmp(*argv,"-white"))
      thegame.wallyColor = WHITE;

    // Display program name and version and exit ?
    else if((0==strcmp(*argv,"-v")) || (0==strcmp(*argv,"--version"))) {
      cout << progname << endl << proginfo << endl << "Version " << version << endl;
      exit( EXIT_SUCCESS );
    }
    // Display short help and exit ?
    else if((0==strcmp(*argv,"-h")) || (0==strcmp(*argv,"--help"))) {
      cout << progname << endl << proginfo << endl << "Version ";
      cout << version << endl << wallyHelp;
      exit( EXIT_SUCCESS );
    }
  }

  if (playgtp){
       if ( gtp() ) // GTP was called, did it quit or abort?
         panic("GTP aborted unexpectedly");
  }
  else {            // play a game on the console
              
/*Make a brief explanation.*/
       cout << "This program plays (poorly) the ancient game of Go, also \n";
       cout << "known as wei-chi." << endl;

/*Play the game.*/
       initgame();

       int rvalue = PLAY_MOVE;  /*return values from enemymove() or mymove()*/

       // Decide who plays first
       if(thegame.blackHandicap < 2)
         thegame.pla= BLACK;
       else
         thegame.pla= WHITE;

       // Start playing
       do {
         if(thegame.pla==thegame.wallyColor)
           rvalue=mymove();
         else
           rvalue=enemymove();
       }
       while(rvalue!=BOTHPASS && rvalue!=RESIGN);

       score();
  }
  exit( EXIT_SUCCESS );         // Game over!
}
